/////////////////////////////////////////////////////////////////////////////////////
//
//  Code written to run off a CFA-10006 which uses an ATMEGA2561 running at 8 MHz.
//
/////////////////////////////////////////////////////////////////////////////////////
//#define F_CPU 8000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include "images.h"

// control lines
#define	LCD_RS	  PC0
#define	LCD_CS1	  PC1
#define	LCD_CS2	  PC2
#define LED_1     PD1
#define LED_2     PD2

// macros for control lines
#define CLR_RS PORTC &= ~(1<<LCD_RS);
#define SET_RS PORTC |=  (1<<LCD_RS);

#define CLR_CS1 PORTC &= ~(1<<LCD_CS1);
#define SET_CS1 PORTC |=  (1<<LCD_CS1);

#define CLR_CS2 PORTC &= ~(1<<LCD_CS2);
#define SET_CS2 PORTC |=  (1<<LCD_CS2);

#define LED1_OFF PORTD &= ~(1 << LED_1);
#define LED1_ON  PORTD |=  (1 << LED_1);
#define LED2_OFF PORTD &= ~(1 << LED_2);
#define LED2_ON  PORTD |=  (1 << LED_2);

#define uint8 unsigned char
#define uint16 unsigned int
#define uint32 unsigned long

volatile unsigned char display[4][122];  // display

// ************************************************************************* //
void write_command(unsigned char command)
{
	PORTA = command;// write the command to the port

	CLR_RS;       // set to write to the control registers

	_delay_us(10);
	SET_CS1;       
	SET_CS2;       
	_delay_ms(2);
	CLR_CS1;       
	CLR_CS2;       
	_delay_ms(2);
}
// ************************************************************************* //
void write_data_left(unsigned char data)
{
	PORTA = data;// write the command to the port

	SET_RS;       // set to write to the control registers

	_delay_us(10);
	SET_CS1;       // clock in the command
	_delay_ms(2);
	CLR_CS1;       // bring write line low
	_delay_ms(2);
}
// ************************************************************************* //
void write_data_right(unsigned char data)
{
	PORTA = data;// write the command to the port

	SET_RS;       // set to write to the control registers

	_delay_us(10);
	SET_CS2;       // clock in the command
	_delay_ms(2);
	CLR_CS2;       // bring write line low
	_delay_ms(2);
}
// ************************************************************************* //
void clear_display()
{
	int row, col;

	for(row = 0; row < 4; row++)
	{
		write_command(0x00);
		write_command(0xB8|row);

		//Talk to controller 1
		for(col=0; col < 61; col++)
		{
			write_data_left(0x00);
		}

		//Talk to controller 2
		for(col=61; col < 122; col++)
		{
			write_data_right(0x00);
		}
	}
	_delay_ms(15);
}
// ************************************************************************* //
void fill_display()
{
	int row, col;

	for(row=0;row<4;row++)
	{
		write_command(0x00);
		write_command(0xB8|row);

		//Talk to controller 1
		for(col=0;col<=60;col++)
		{
			write_data_left(0xFF);
		}

		//Talk to controller 2
		for(col=61;col<=121;col++)
		{
			write_data_right(0xFF);
		}
	}
	_delay_ms(15);
}
// ************************************************************************* //
void goodnight()
{
	write_command(0xAE);
	write_command(0xA5);
}
// ************************************************************************* //
void wakey_wakey()
{
	write_command(0xAF);
	write_command(0xA4);
}
// ************************************************************************* //
void initialize_display()
{
	SET_RS;
	SET_CS1;
	SET_CS2;
	PORTA=0x00;

	write_command(0xAE);  //Display off
	write_command(0xC0);  //Set Start Line 0
	write_command(0xA4);  //Set static drive off
	write_command(0x00);  //Set Column Address counter
	write_command(0xB8);  //Set Page Address
	write_command(0xA9);  //Set Duty Cycle to 1/32
	write_command(0xA0);  //Select ADC
	write_command(0xEE);  //read modify write off
	write_command(0xAF);  //Display on
}

int displayImage()
{
	//Send data command
	write_command(0x0C);

	for(int row=0;row<4;row++)
	{
		write_command(0x00);		//Set X address
		write_command(0xB8|row);	//Set Y address

		//Talk to controller 1
		for(int col=0;col<=60;col++)
		{
			write_data_left(display[row][col]);
			//write_data_left(cfag12232k[row][col]);
			//write_data_left(tractor[row][col]);
		}

		//Talk to controller 2
		for(int col=61;col<=121;col++)
		{
			write_data_right(display[row][col]);
			//write_data_right(cfag12232k[row][col]);
		}
	}
}

void main(void)
{
	DDRA  = 0xFF;  // all pins as outputs
	DDRC  = 0xFF;  // all pins as outputs
	DDRD |= 0x06;  // Switches as inputs, LEDs as outputs

	PORTA = 0xFF;  // data lines low
	PORTC = 0xFF;  // control and other lines high
	PORTD = 0xF0;  // pull ups on the keys, remaining pins low

	_delay_ms(100);

	// Initialize the display
	initialize_display();

	while (1)
	{
		clear_display();
		_delay_ms(5000);
/*
		fill_display();
		_delay_ms(5000);
*/

		memcpy_P(display, cfag12232k, sizeof(cfag12232k));
		displayImage();
		_delay_ms(5000);
		
		memcpy_P(display, tractor, sizeof(tractor));
		displayImage();
		_delay_ms(5000);

/*
		goodnight();
		_delay_ms(1000);
		wakey_wakey();
		_delay_ms(1000);
*/
	}
	return 0;
}
